@@ -7,7 +7,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Mojo::Log;
 use Config::JSON::Enhanced;
@@ -21,6 +21,31 @@ use Android::ElectricSheep::Automator;
 use Android::ElectricSheep::Automator::ScreenLayout;
 use Android::ElectricSheep::Automator::XMLParsers;
 
+my $_DEFAULT_CONFIG = <<'EODC';
+</* $VERSION = '0.01'; */>
+</* comments are allowed */>
+</* and <% vars %> and <% verbatim sections %> */>
+{
+	"Android::ElectricSheep::Automator" : {
+		"adb" : {
+			"path-to-executable" : "/usr/local/android-sdk/platform-tools/adb"
+		},
+		"debug" : {
+			"verbosity" : 0,
+			</* cleanup temp files on exit */>
+			"cleanup" : 1
+		},
+		"logger" : {
+			</* log to file if you uncomment this */>
+			</* "filename" : "..." */>
+		}
+		</* config for our plugins (each can go to separate file also) */>
+	},
+	"Android::ElectricSheep::Automator::Plugins::Viber" : {
+	}
+}
+EODC
+
 sub new {
 	my $class = ref($_[0]) || $_[0];
 	my $params = $_[1] // {};
@@ -124,7 +149,9 @@ sub confighash {
 	print STDOUT "${whoami} (via $parent), line ".__LINE__." : called ...\n";
 
 	# the confighash must contain a mother section: 'Android::ElectricSheep::Automator'
-	# and a child section, e.g. 'Android::ElectricSheep::Automator::Plugins::Viber'
+	# and a child section,
+	# e.g. 'Android::ElectricSheep::Automator::Plugins::Viber'
+	# check for both here:
 	for ('Android::ElectricSheep::Automator', $self->child_class){
 		if( ! exists($m->{$_}) || ! defined($m->{$_}) || (ref($m->{$_})ne'HASH') ){ print STDERR perl2dump($m)."${whoami} (via $parent), line ".__LINE__." : error, configuration (see above) does not have key '$_' or its value is not a HASHref.\n"; return undef }
 	}
@@ -136,6 +163,13 @@ sub confighash {
 		if( ! exists($m->{$x}->{$_}) || ! defined($m->{$x}->{$_}) ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, configuration does not have key '$x'->'$_'.\n"; return undef }
 	}
 
+	$x = $self->child_class;
+	# we are storing specified confighash but first check it for some fields
+	# required fields:
+	#for ('debug', 'logger'){
+	#	if( ! exists($m->{$x}->{$_}) || ! defined($m->{$x}->{$_}) ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, configuration does not have key '$x'->'$_'.\n"; return undef }
+	#}
+
 	# ok!
 	$self->{'_private'}->{'confighash'} = $m;
 	return $m
@@ -179,7 +213,19 @@ sub init_module_specific {
 		$confighash = $params->{'confighash'};
 		# this sets the confighash and checks it too
 		if( ! defined $self->confighash($confighash) ){ print STDERR __PACKAGE__." (via ".$self->child_class.") : ${whoami} (via $parent), line ".__LINE__." : error, call to ".'confighash()'." has failed.\n"; return 1 }
-	} else { print STDERR __PACKAGE__." (via ".$self->child_class.") : ${whoami} (via $parent), line ".__LINE__." : error, one of 'configfile' or 'confighash' must be specified in the input parameters unless mother's config contains a section for this package (".$self->child_class.") : .\n"; return 1 }
+	} else {
+		# use default config
+		$confighash = Config::JSON::Enhanced::config2perl({
+			'string' => $_DEFAULT_CONFIG,
+			'commentstyle' => 'custom(</*)(*/>)',
+			'tags' => ['<%','%>'],
+			'variable-substitutions' => {
+				'SCRIPTDIR' => Cwd::abs_path($FindBin::Bin),
+			},
+		});
+		if( ! defined $confighash ){ print STDERR $_DEFAULT_CONFIG."\n\n".__PACKAGE__."${whoami} (via $parent), line ".__LINE__." : error, failed to parse default configuration string, above.\n"; return undef }
+		if( ! defined $self->confighash($confighash) ){ print STDERR __PACKAGE__."${whoami} (via $parent), line ".__LINE__." : error, call to ".'confighash()'." has failed.\n"; return 1 }
+	}
 
 	if( exists($params->{'mother'}) && defined($params->{'mother'}) ){
 		$self->{'_private'}->{'mother'} = $params->{'mother'};
@@ -260,53 +306,54 @@ Android::ElectricSheep::Automator::Plugins::Base - The great new Android::Electr
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 
 =head1 SYNOPSIS
 
-Quick summary of what the module does.
+This is the parent class of all L<Android::ElectricSheep::Automator>
+plugins. You do not need to override anything except perhaps the
+constructor if you are going to be needing extra input parameters
+to it. There is already one plugin provided L<Android::ElectricSheep::Automator::Plugins::Viber>
+which can serve as an example for creating new plugins.
+It is as simple as this:
 
-Perhaps a little code snippet.
+    package Android::ElectricSheep::Automator::Plugins::MyNewPlugin;
 
-    use Android::ElectricSheep::Automator::Plugins::Base;
+    use parent 'Android::ElectricSheep::Automator::Plugins::Base';
 
-    my $foo = Android::ElectricSheep::Automator::Plugins::Base->new();
+    sub new {
+            my ($class, $params) = @_;  
+            my $self = $class->SUPER::new({
+                    %$params,
+                    'child-class' => $class,
+            });
+            # add some extra internal fields, e.g. the name of the app
+            # we are dealing with
+            $self->{'_private'}->{'appname'} = 'com.viber.voip';
+    
+            return $self;
+    }
+    # new methods
+    # getter of the app name
+    sub appname { return $_[0]->{'_private'}->{'appname'} }
+    # any methods for controlling the app, e.g.
+    sub open_viber_app {
+        my ($self, $params) = @_;
+        ...
+    }
     ...
 
-=head1 RUNNING AN EMULATOR
-
-List all emulators currently available with C<emulator -list-avds>
-It outputs something like:
-
-    Pixel_2_API_27_x86_
-    Pixel_2_API_30_x86_
-
-Run an emulator with C<emulator -avd Pixel_2_API_30_x86_>
-
-=head1 USING YOUR REAL SMARTPHONE
-
-Connect your smartphone with a USB cable to your computer.
-Become developer with this procedure:
-
-Go to C<Settings->System->About Phone>
-Tap on C<Build Number> 7 times!!!
-Enter your phone pin and you are in developer mode.
-
-
-=head1 EXPORT
-
-A list of functions that can be exported.  You can delete this section
-if you don't export anything, such as for a purely object-oriented module.
-
-=head1 SUBROUTINES/METHODS
-
-=head2 function1
-
-
-
-=head2 function2
+Then use the plugin as:
 
+   use Android::ElectricSheep::Automator::Plugins::MyNewPlugin;
+   my $vib = Android::ElectricSheep::Automator::Plugins::MyNewPlugin->new({
+     configfile=>'config/plugins/viber.conf',
+     'device-is-connected' => 1
+   });
+   $vib->open_viber_app();
+   $vib->send_message({recipient=>'My Notes', message=>'hello%sMonkees'});
+   $vib->close_viber_app();
 
 =head1 AUTHOR
 
