@@ -3,12 +3,15 @@ package Android::ElectricSheep::Automator::Plugins::Viber;
 use strict;
 use warnings;
 
-use lib ($FindBin::Bin, 'blib/lib');
+#use lib ($FindBin::Bin, 'blib/lib');
 
 use parent 'Android::ElectricSheep::Automator::Plugins::Base';
-use Android::ElectricSheep::Automator::XMLParsers;
+
+use Time::HiRes qw/usleep/;
 use Data::Roundtrip qw/perl2dump no-unicode-escape-permanently/;
 
+use Android::ElectricSheep::Automator::XMLParsers;
+
 sub new {
 	my ($class, $params) = @_;
 	my $self = $class->SUPER::new({
@@ -73,12 +76,18 @@ sub navigate_to_viber_home_activity {
 	my ($ui, $dom, $xc, $asel, @nodes, $N, $node, $boundstr, $bounds);
 
 	my $repeats = 3;
+	my $repeatsUI = 3;
 	ONBACKARROW:
 	while(--$repeats > 0){
 		# we assume the app is open and at the foreground
 		# get the UI
 		$outfile = defined($outbase) ? $outbase.'_main.xml' : undef;
-		$ui = $self->mother->dump_current_screen_ui({'filename'=>$outfile});
+		do {
+			$ui = $self->mother->dump_current_screen_ui({'filename'=>$outfile});
+			usleep(0.75);
+		} while( ($repeatsUI-- > 0) && ! defined($ui) );
+		if( ! defined $ui ){ $log->error("${whoami} (via $parent), line ".__LINE__." : error, failed to dump the UI, call to ".'dump_current_screen_ui()'." has failed after a number of repeats. I am not sure what the problem is, most likely a race condition ..."); return undef }
+
 		$dom = $ui->{'XML::LibXML'};
 		$xc = $ui->{'XML::LibXML::XPathContext'};
 		$asel = '//node'
@@ -105,7 +114,7 @@ sub navigate_to_viber_home_activity {
 		# click it
 		if( $verbosity > 0 ){ $log->info("${whoami} (via $parent), line ".__LINE__." : clicking the back-arrow ...") }
 		if( $self->mother->tap({'bounds' => $bounds}) ){ $log->error("${whoami} (via $parent), line ".__LINE__." : error, failed to tap on $bounds."); return 1 }
-		sleep(1);
+		usleep(0.75);
 	}
 
 	if( $verbosity > 0 ){ $log->info("${whoami} (via $parent), line ".__LINE__." : hopefully we are now at the very first screen of the app.") }
@@ -127,16 +136,24 @@ sub send_message {
 	# do everything except clicking the send button
 	my $mock = exists($params->{'mock'}) ? $params->{'mock'} : 0;
 
-	my ($ui, $dom, $xc, $asel, @nodes, $N, $node, $boundstr, $bounds);
+	my ($dom, $xc, $asel, @nodes, $N, $node, $boundstr, $bounds);
 
 	if( $self->navigate_to_viber_home_activity({'outbase'=>$outbase}) ){ $log->error("${whoami} (via $parent), line ".__LINE__." : error, call to ".'navigate_to_viber_home_activity()'." has failed."); return undef }
+	usleep(1.5);
 
 	# we assume the app is open and at the foreground
 	# get the UI
 	$outfile = defined($outbase) ? $outbase.'_main.xml' : undef;
 	# there is always the chance you get 'Is taking too long'...
 	# and all will fail TODO!
-	$ui = $self->mother->dump_current_screen_ui({'filename'=>$outfile});
+	my $ui;
+	my $repeatsUI = 3;
+	do {
+		$ui = $self->mother->dump_current_screen_ui({'filename'=>$outfile});
+		usleep(0.75);
+	} while( ($repeatsUI-- > 0) && ! defined($ui) );
+	if( ! defined $ui ){ $log->error("${whoami} (via $parent), line ".__LINE__." : error, failed to dump the UI, call to ".'dump_current_screen_ui()'." has failed after a number of repeats. I am not sure what the problem is, most likely a race condition ..."); return undef }
+
 	$dom = $ui->{'XML::LibXML'};
 	$asel = '//node[@text="Chats" and @resource-id="com.viber.voip:id/bottomBarItemTitle"]';
 	@nodes = $dom->findnodes($asel);
@@ -180,7 +197,7 @@ sub send_message {
 	$bounds = [[$1,$2],[$3,$4]];
 	# click it
 	if( $self->mother->tap({'bounds' => $bounds}) ){ $log->error("${whoami} (via $parent), line ".__LINE__." : error, failed to tap on $bounds."); return undef }
-	sleep(1);
+	usleep(1.5);
 
 	# Put the text into the text-edit Message... (note: ... is unicode ellipses something)
 	# get the UI
@@ -234,7 +251,7 @@ sub send_message {
 	} else {
 		$log->warn("${whoami} (via $parent), line ".__LINE__." : The 'Send' button was not clicked because mock is ON.")
 	}
-	sleep(0.8);
+	usleep(0.8);
 
 	return {};
 }
@@ -250,7 +267,7 @@ Android::ElectricSheep::Automator::Plugins::Viber - Control the Viber app from y
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 WARNING
 
